SystemOrganization addCategory: #MonticelloPackageInstaller!

Exception subclass: #MCPStopInstallException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloPackageInstaller'!

!MCPStopInstallException methodsFor: 'actions' stamp: 'Noury 12/20/2006 17:54'!
defaultAction
	Transcript cr; show: 'Install stopped'! !

Object subclass: #MCPInstaller
	instanceVariableNames: 'packageBaseNames monticelloBrowser defaultRepository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloPackageInstaller'!

!MCPInstaller commentStamp: 'Noury 12/20/2006 10:29' prior: 0!
A MCPInstaller is an object responsible for installing one or more monticello packages from a repository. It is particulary usefull for dealing with "meta-dependencies", i.e. dependencies that require extending monticello and using the extension to load some other package. 

A typical example where MCPInstaller is required is for loading a package with BootStrappedClasses (classes that are instance of their selves) or MetaclassTalk explicit metaclasses. 

The most appropriate way of using MCPInstaller is to create a new package with a subclass with an initialize class method that loads, first the packages extending monticello and then the other "business" pacakges that are based on this. Instead of loading the "business" packages directly, one will have to load the installer package instead.

!

!MCPInstaller class methodsFor: 'installing' stamp: 'Noury 12/20/2006 18:39'!
installAll: packageBaseNamesArray
	self new installAll: packageBaseNamesArray! !

!MCPInstaller class methodsFor: 'installing' stamp: 'Noury 12/20/2006 13:25'!
installAll: packageBaseNamesArray from: repository 
	self new installAll: packageBaseNamesArray from: repository! !

!MCPInstaller class methodsFor: 'installing' stamp: 'Noury 12/20/2006 10:14'!
loadFromCacheInOrderLatestVersionsOf: packageBaseNamesArray 
	self new loadFromCacheInOrderLatestVersionsOf: packageBaseNamesArray 
! !

!MCPInstaller methodsFor: 'user interaction' stamp: 'Noury 12/20/2006 17:41'!
addRepositoryThanInstall: packageBaseName 
	| repository |
	repository := self monticelloBrowser newRepository.
	repository
		ifNil: [^ self alternativesForPackage: packageBaseName menuTitle: 'No package provided'].
	self monticelloBrowser addRepository: repository.
	self install: packageBaseName from: repository ! !

!MCPInstaller methodsFor: 'user interaction' stamp: 'Noury 12/20/2006 18:13'!
alternativesForPackage: packageBaseName menuTitle: aString 
	| menu |
	menu := MenuMorph entitled: aString.
	menu
		add: 'Load from an existing repository'
		target: self
		selector: #requestRepositoryThanInstall:
		argument: packageBaseName;
		
		add: 'Load from a new repository'
		target: self
		selector: #addRepositoryThanInstall:
		argument: packageBaseName;
		
		add: 'Skip package'
		target: self
		selector: #skipPackage:
		argument: packageBaseName;
		
		add: 'Stop install'
		target: self
		selector: #stopInstall.
	menu invokeModal.
	menu selectedItem
		ifNil: ["Default action is skip"
			self skipPackage: packageBaseName]! !

!MCPInstaller methodsFor: 'user interaction' stamp: 'Noury 12/20/2006 17:40'!
alternativesForPackage: packageBaseName notAvailableIn: repository 
	| title |
	title := packageBaseName , '\is not available in repository\' withCRs , repository description.
	self alternativesForPackage: packageBaseName menuTitle: title 
	! !

!MCPInstaller methodsFor: 'accessing' stamp: 'Noury 12/20/2006 12:15'!
defaultRepository
	^defaultRepository! !

!MCPInstaller methodsFor: 'accessing' stamp: 'Noury 12/20/2006 12:15'!
defaultRepository: anObject
	defaultRepository := anObject! !

!MCPInstaller methodsFor: 'installing' stamp: 'Noury 12/20/2006 12:32'!
findLatestVersionNameOf: packageBaseName in: aMCRepository 
	| allPacakgeVersionNames versionNumbers maxVersionIndex |
	allPacakgeVersionNames := aMCRepository allVersionNames
				select: [:versionName | versionName beginsWith: packageBaseName].
	allPacakgeVersionNames ifEmpty: [^nil].
	versionNumbers := allPacakgeVersionNames
				collect: [:versionName | (versionName copyAfterLast: $.) asNumber].
	maxVersionIndex := versionNumbers indexOf: versionNumbers max.
	^ allPacakgeVersionNames at: maxVersionIndex! !

!MCPInstaller methodsFor: 'user interaction' stamp: 'Noury 12/20/2006 17:34'!
install: packageBaseName 
	self install: packageBaseName from: self defaultRepository.
	! !

!MCPInstaller methodsFor: 'user interaction' stamp: 'Noury 12/20/2006 17:56'!
install: packageBaseName from: repository 
	| version |
	version := self loadLatestVersionOf: packageBaseName from: repository.
	version
		ifNotNil: [^ self].
	self alternativesForPackage: packageBaseName notAvailableIn: repository! !

!MCPInstaller methodsFor: 'user interaction' stamp: 'Noury 12/20/2006 23:14'!
installAll
	[self packageBaseNames
		do: [:packageBaseName | self install: packageBaseName]]
		on: MCPStopInstallException
		do: [:ex| ^ self]! !

!MCPInstaller methodsFor: 'user interaction' stamp: 'Noury 12/20/2006 22:56'!
installAll: packageBaseNamesArray 
	| repository |
	repository := self retreiveRepositoryFromContext
				ifNil: [self requestRepository: 'Select the default repository'].
	self installAll: packageBaseNamesArray from: repository! !

!MCPInstaller methodsFor: 'user interaction' stamp: 'Noury 12/20/2006 22:49'!
installAll: packageBaseNamesArray from: repository 
	self defaultRepository: repository.
	self packageBaseNames: packageBaseNamesArray.
	self installAll! !

!MCPInstaller methodsFor: 'installing' stamp: 'Noury 12/20/2006 13:25'!
loadFromCacheInOrderLatestVersionsOf: packageBaseNamesArray 
	self installAll: packageBaseNamesArray from: MCCacheRepository default! !

!MCPInstaller methodsFor: 'installing' stamp: 'Noury 12/20/2006 12:53'!
loadLatestVersionOf: packageBaseName from: repository 
	| latestVersionName latestVersion |
	latestVersionName := self findLatestVersionNameOf: packageBaseName in: repository.
	latestVersionName ifNil: [^nil].
	latestVersion := repository versionFromFileNamed: latestVersionName , '.mcz'.
	^latestVersion load! !

!MCPInstaller methodsFor: 'accessing' stamp: 'Noury 12/20/2006 11:47'!
monticelloBrowser
	monticelloBrowser
		ifNil: [self monticelloBrowser: self monticelloBrowserClass new].
	^ monticelloBrowser! !

!MCPInstaller methodsFor: 'accessing' stamp: 'Noury 12/20/2006 11:46'!
monticelloBrowser: anObject
	monticelloBrowser := anObject! !

!MCPInstaller methodsFor: 'accessing' stamp: 'Noury 12/20/2006 11:47'!
monticelloBrowserClass
	^MCWorkingCopyBrowser! !

!MCPInstaller methodsFor: 'accessing' stamp: 'Noury 12/20/2006 11:30'!
packageBaseNames
	^packageBaseNames! !

!MCPInstaller methodsFor: 'accessing' stamp: 'Noury 12/20/2006 11:30'!
packageBaseNames: anObject
	packageBaseNames := anObject! !

!MCPInstaller methodsFor: 'accessing' stamp: 'Noury 12/20/2006 11:46'!
repositories
	^ self monticelloBrowser repositories! !

!MCPInstaller methodsFor: 'accessing' stamp: 'Noury 12/20/2006 11:46'!
repositoryDescriptions
	^ self monticelloBrowser repositoryList! !

!MCPInstaller methodsFor: 'user interaction' stamp: 'Noury 12/20/2006 23:12'!
requestRepository: requestString 
	| selectionIndex |
	[selectionIndex := PopUpMenu withCaption: requestString chooseFrom: self repositoryDescriptions.
	selectionIndex = 0] whileTrue: [self inform: 'Please select a package'].
	^ self repositories at: selectionIndex! !

!MCPInstaller methodsFor: 'user interaction' stamp: 'Noury 12/20/2006 17:35'!
requestRepositoryThanInstall: packageBaseName 
	| repository |
	repository := self requestRepository: 'Repository to install\' withCRs , packageBaseName.
	self install: packageBaseName from: repository! !

!MCPInstaller methodsFor: 'repository management' stamp: 'Noury 12/20/2006 23:19'!
retreiveRepositoryFromContext
	| context repositoryInspector |
	context := thisContext.
	[context receiver
		ifNil: [^ nil].
	context receiver isKindOf: MCFileRepositoryInspector]
		whileFalse: [context := (context isKindOf: BlockContext)
						ifTrue: [context home]
						ifFalse: [context sender]].
	repositoryInspector := context receiver.
	^ repositoryInspector instVarNamed: #repository! !

!MCPInstaller methodsFor: 'user interaction' stamp: 'Noury 12/20/2006 17:56'!
skipPackage: packageBaseName 
	self inform: 'Skipping package ' , packageBaseName! !

!MCPInstaller methodsFor: 'user interaction' stamp: 'Noury 12/20/2006 17:43'!
stopInstall
	MCPStopInstallException signal! !
